//Sprite.cpp
#include "Sprite.h"

Sprite::Sprite( IDirect3DDevice9* pDevice )
{
    D3DXCreateSprite( pDevice, &m_pSprite );
}
//-----------------------------------------------------------------
Sprite::~Sprite()
{
	Release();
}
//-----------------------------------------------------------------
void Sprite::Release()
{
	if( m_pSprite )
	{
		m_pSprite->Release();
		m_pSprite = NULL;
	}
}
//-----------------------------------------------------------------
void Sprite::Begin( DWORD flags )
{
    m_pSprite->Begin( flags );
}
//-----------------------------------------------------------------
void Sprite::End()
{
    m_pSprite->End();
}
//-----------------------------------------------------------------
void Sprite::Draw( Texture2D* pTexture, const POINT& pos, D3DCOLOR color )
{
    Draw( pTexture, pos, pTexture->GetRect(), color );
}
//-----------------------------------------------------------------
void Sprite::Draw( Texture2D* pTexture, const POINT& pos, const RECT& srcRect, D3DCOLOR color )
{
    m_pSprite->Draw( pTexture->GetTexture(), &srcRect, &D3DXVECTOR3(0, 0, 0), 
		&D3DXVECTOR3( (FLOAT)pos.x, (FLOAT)pos.y, 0), color);
}
//-----------------------------------------------------------------
void Sprite::Draw( Texture2D* pTexture, const POINT& pos, const D3DXMATRIX& transMatrix, const RECT& surRect, D3DCOLOR color)
{
    m_pSprite->SetTransform( &transMatrix );
    Draw( pTexture, pos, surRect, color );
}
//-----------------------------------------------------------------
void Sprite::Draw( Texture2D* pTexture, const POINT& pos, const POINT& size, const RECT& surRect, D3DCOLOR color)
{
    D3DXMATRIX	transMatrix;
    float		scaleX, scaleY;

    scaleX = (float)size.x / (float) ( surRect.right - surRect.left );
    scaleY = (float)size.y / (float) ( surRect.bottom - surRect.top );
    D3DXMatrixScaling( &transMatrix, scaleX, scaleY, 0 );
    Draw( pTexture, pos, transMatrix, surRect, color);
}
//-----------------------------------------------------------------
void Sprite::Draw( Texture2D* pTexture, const RECT& desRect, const RECT& surRect, D3DCOLOR color )
{
    POINT	pos;
    pos.x = desRect.left;
    pos.y = desRect.top;

    POINT	size;
    size.x = desRect.right - desRect.left;
    size.y = desRect.bottom - desRect.top;
    Draw( pTexture, pos, size, surRect, color );
}
//-----------------------------------------------------------------
void Sprite::Draw( Texture2D* pTexture, const RECT& desRect, D3DCOLOR color )
{
    Draw( pTexture, desRect, pTexture->GetRect(), color );
}
//-----------------------------------------------------------------


