#include <windows.h>
#include <time.h>
#include <d3d9.h>
#include <d3dx9.h>

#include "dxfunc.h"

#pragma comment (lib, "dxguid.lib")
#pragma comment (lib, "d3d9.lib")
#pragma comment (lib, "d3dx9.lib")
#pragma comment (lib, "winmm.lib")

#define		PI					(3.1415926)
#define		DEGtoRAD(x)			(float)((float)x * PI / 180.0f)

float		g_gridStride = 7;


//ؽ ü Ÿ 
struct CUSTOMVERTEX { 
	float x, y, z; 
	DWORD color; 
	float tu, tv;
};

#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1)

LPDIRECT3DTEXTURE9 g_pTexture = NULL;


LPDIRECT3D9             g_d3d        = NULL; 
LPDIRECT3DDEVICE9       g_dxDevice = NULL; 
LPDIRECT3DVERTEXBUFFER9 g_VertexBuffer = NULL; 
D3DPRESENT_PARAMETERS	g_d3dpp; 
bool g_bIsFullScreen = false;	

//ʱⰪ ׿ ؼ ־ ũ ǹ̰ .
D3DDISPLAYMODE  g_d3ddm = {640, 480, 60, D3DFMT_A8R8G8B8};

template<class T> void _RELEASE_(T t)
{
	if( t )
	{
		t->Release();
		t = 0;
	}
}

template<class T> void _DELETE_(T t)
{
	if( t )
	{
		delete t;
		t = 0;
	}
}

//main.cpp 
extern HWND   g_hRenderWnd;

#include "Texture2D.h"
#include "Sprite.h"

Sprite*		g_pSprite = NULL;
Texture2D*	g_pTexture2D = NULL;


HRESULT InitD3D(HWND hWnd)
{
	ZeroMemory (&g_d3dpp, sizeof(g_d3dpp));

    if( NULL == ( g_d3d = Direct3DCreate9( D3D_SDK_VERSION ) ) )
        return E_FAIL;

	//ؽ ó 
	D3DCAPS9 caps;
	D3DDEVTYPE deviceType = D3DDEVTYPE_HAL;
	g_d3d->GetDeviceCaps(D3DADAPTER_DEFAULT, deviceType, &caps);
	int vertexProcessing = 0;
	if( caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT )
		vertexProcessing = D3DCREATE_HARDWARE_VERTEXPROCESSING;
	else
		vertexProcessing = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

	if( FAILED( g_d3d->GetAdapterDisplayMode (D3DADAPTER_DEFAULT, &g_d3ddm)))
			return E_FAIL;

    g_d3dpp.Windowed = !g_bIsFullScreen;
    g_d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    g_d3dpp.BackBufferFormat = g_d3ddm.Format;
	g_d3dpp.hDeviceWindow              = g_hRenderWnd;
	g_d3dpp.EnableAutoDepthStencil     = true; 
	g_d3dpp.AutoDepthStencilFormat     = D3DFMT_D24S8;

    if(FAILED(g_d3d->CreateDevice (D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, 
								   vertexProcessing, 
								   &g_d3dpp, &g_dxDevice)))
	{
		return E_FAIL;
	}

	//g_dxDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
	//g_dxDevice->SetRenderState( D3DRS_AMBIENT, 0xffffffff );
	g_dxDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
	//g_dxDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW );
	g_dxDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE );

	 D3DXCreateTextureFromFile( g_dxDevice, "grim.jpg", &g_pTexture );

	//Ʈ ʱȭ
	g_pSprite = new Sprite( g_dxDevice );
	g_pTexture2D = new Texture2D;
	g_pTexture2D->LoadTexture( g_dxDevice, "grim.jpg" );


	return S_OK;
}
//-----------------------------------------------------------------------------
void SetupCamera(HWND hWnd)
{
    D3DXMATRIX   matProj;
	D3DXMATRIX	 matCameraView;
	D3DXVECTOR3  cameraTarget;
    D3DXVECTOR3  cameraPos;

	// Set where camera is viewing from
	float fDist = g_gridStride;
	cameraPos.x = 0.0f;
	cameraPos.y = -fDist;
	cameraPos.z = fDist;
	// Set what camera is looking at
	cameraTarget.x = 0.0f;
	cameraTarget.y = 0.0f;
	cameraTarget.z = 0.0f;

	// Setup Camera View
	D3DXMatrixLookAtRH( &matCameraView, 
						&cameraPos, 
						&cameraTarget, 
						&D3DXVECTOR3( 0.0f, 0.0f, 1.0f ) );
	// Tell the device to use the camera view for the viewport
	g_dxDevice->SetTransform( D3DTS_VIEW, &matCameraView );

	// Setup Projection 
	RECT rect;
	GetClientRect(hWnd, &rect);
	float width = (float)rect.right, height = (float)rect.bottom;
	D3DXMatrixPerspectiveFovRH( &matProj, D3DX_PI/4, width/height,  1.0f, g_gridStride * 10.0f);
	// Tell the device to use the above matrix for projection
	g_dxDevice->SetTransform( D3DTS_PROJECTION, &matProj );
}

void DestroyDirect()
{
	delete g_pTexture2D; 
	delete g_pSprite;

	_RELEASE_<LPDIRECT3DTEXTURE9>(g_pTexture);
	_RELEASE_<LPDIRECT3DVERTEXBUFFER9>(g_VertexBuffer);
	_RELEASE_<LPDIRECT3DDEVICE9>(g_dxDevice);
	_RELEASE_<LPDIRECT3D9>(g_d3d);
}

bool RenderInit()
{
	float	fSizeX = 5.0f;
	float	fSizeY = 5.0f;
	
	CUSTOMVERTEX	vertex[4];

	// Create the geometry
	vertex[0].x = -fSizeX/2;
	vertex[0].y = 0.0f;
	vertex[0].z = -fSizeY/2;
	vertex[0].color = 0xffffff;
	vertex[0].tu = 0.0f;
    vertex[0].tv = 1.0f;

	vertex[1].x = fSizeX/2;
	vertex[1].y = 0.0f;
	vertex[1].z = -fSizeY/2;
	vertex[1].color = 0xffffff;
	vertex[1].tu = 1.0f;
    vertex[1].tv = 1.0f;


	vertex[2].x = -fSizeX/2;
	vertex[2].y = 0.0f;
	vertex[2].z = fSizeY/2;
	vertex[2].color = 0xffffff;
	vertex[2].tu = 0.0f;
    vertex[2].tv = 0.0f;

	vertex[3].x = fSizeX/2;
	vertex[3].y = 0.0f;
	vertex[3].z = fSizeY/2;
	vertex[3].color = 0xffffff;
	vertex[3].tu = 1.0f;
    vertex[3].tv = 0.0f;

	g_dxDevice->CreateVertexBuffer (4*sizeof(CUSTOMVERTEX), 0, D3DFVF_CUSTOMVERTEX, D3DPOOL_MANAGED, &g_VertexBuffer, NULL);
	
	VOID* pVertices;
	g_VertexBuffer->Lock (0, sizeof(vertex), &pVertices, 0);
    memcpy (pVertices, vertex, sizeof(vertex));
    g_VertexBuffer->Unlock();

	return true;
}

HRESULT ResetDeviceObjects()
{
	HRESULT hr;

    // Release all vidmem objects
    //if( FAILED( hr = InvalidateDeviceObjects() ) )
    //    return hr;

	// Reset the device
	if( FAILED( hr = g_dxDevice->Reset( &g_d3dpp ) ) )
		return hr;

	return S_OK;
}

HRESULT RenderLoop()
{
    HRESULT hr;

    // Test the cooperative level to see if it's okay to render
    if( FAILED( hr = g_dxDevice->TestCooperativeLevel() ) )
    {
        // If the device was lost, do not render until we get it back
        if( D3DERR_DEVICELOST == hr )
            return S_OK;

        // Check if the device needs to be resized.
        if( D3DERR_DEVICENOTRESET == hr )
        {
            // If we are windowed, read the desktop mode and use the same format for
            // the back buffer
            if( !g_bIsFullScreen )
            {
				g_d3d->GetAdapterDisplayMode (D3DADAPTER_DEFAULT, &g_d3ddm);
                g_d3dpp.BackBufferFormat = g_d3ddm.Format;
            }

            if( FAILED( hr = ResetDeviceObjects() ) )
                return hr;
        }
        return hr;
    }

    // Render the scene as normal
    if( FAILED( hr = Render3D() ) )
        return hr;

	return S_OK;
}


HRESULT Render3D ()
{
	//ȸ
	static	float	angle = 0.0f;
	D3DXMATRIX	matWorld;
	D3DXMatrixRotationZ(&matWorld, DEGtoRAD(angle));
	angle += 1.0f;
	g_dxDevice->SetTransform(D3DTS_WORLD, &matWorld);

	//帵
	HRESULT hr;
    g_dxDevice->Clear (0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB (0, 0, 0), 1.0f, 0);
    g_dxDevice->BeginScene ();

	g_dxDevice->SetTexture( 0, g_pTexture );

    g_dxDevice->SetFVF (D3DFVF_CUSTOMVERTEX);
    g_dxDevice->SetStreamSource (0, g_VertexBuffer, 0, sizeof(CUSTOMVERTEX));
    g_dxDevice->DrawPrimitive (D3DPT_TRIANGLESTRIP, 0, 2);

	//ĶƮ 
	D3DXMATRIX spriteMat;
	D3DXMatrixRotationZ( &spriteMat, D3DXToRadian(-15.0f) );
	spriteMat._41 = 100.0f;   
	spriteMat._42 = 100.0f;    
	spriteMat._11 = 0.25f;
	spriteMat._22 = 0.25f;
	//pSprite->SetTransform( &mat );
	

	POINT pos;
	pos.x = 0;
	pos.y = 0;
	g_pSprite->Begin( D3DXSPRITE_ALPHABLEND );
	const D3DXCOLOR WHITE( D3DCOLOR_RGBA(255, 255, 255, 0xa0 ) );
	//g_pSprite->Draw( g_pTexture2D, pos, g_pTexture2D->GetRect(), WHITE );
	g_pSprite->Draw( g_pTexture2D, pos, spriteMat, g_pTexture2D->GetRect(), WHITE );
	g_pSprite->End();

    g_dxDevice->EndScene ();
    hr = g_dxDevice->Present (NULL, NULL, NULL, NULL);
	return hr;
}

